@@ -57,18 +57,12 @@
 
 #include "sox_i.h"
 
-#if defined(HAVE_LAME_LAME_H) || defined(HAVE_LAME_H) || defined(DL_LAME)
-#define HAVE_LAME 1
-#endif
-
-#if defined(HAVE_TWOLAME_H) || defined(DL_TWOLAME)
-  #define HAVE_TWOLAME 1
-#endif
-
-#if defined(HAVE_MAD_H) || defined(HAVE_LAME) || defined(HAVE_TWOLAME)
+#if defined(HAVE_MAD) || defined(HAVE_LAME) || defined(HAVE_TWOLAME)
 
 #ifdef HAVE_MAD_H
 #include <mad.h>
+#elif defined(DL_MAD)
+#include "bit-rot/mad.h"
 #endif
 
 #if defined(HAVE_LAME_LAME_H)
@@ -100,12 +94,15 @@ typedef enum {
 #endif
 
 #ifdef HAVE_TWOLAME_H
-  #include <twolame.h>
+#include <twolame.h>
+#elif defined(DL_LAME)
+typedef struct twolame_options_struct twolame_options;
 #endif
 
 #ifndef HAVE_LIBLTDL
-  #undef DL_LAME
   #undef DL_MAD
+  #undef DL_LAME
+  #undef DL_TWOLAME
 #endif
 
 /* Under Windows, importing data from DLLs is a dicey proposition. This is true
@@ -114,7 +111,7 @@ typedef enum {
  * Sidestep the issue by defining our own mad_timer_zero. This is needed because
  * mad_timer_zero is used in some of the mad.h macros.
  */
-#ifdef HAVE_MAD_H
+#ifdef HAVE_MAD
 #define mad_timer_zero mad_timer_zero_stub
 static mad_timer_t const mad_timer_zero_stub = {0, 0};
 #endif
@@ -122,15 +119,22 @@ static mad_timer_t const mad_timer_zero_stub = {0, 0};
 #define MAXFRAMESIZE 2880
 #define ID3PADDING 128
 
+/* Twolame takes float values as input. */
+#define MP2_TWOLAME_PRECISION   24
+
 /* LAME takes float values as input. */
 #define MP3_LAME_PRECISION   24
 
+/* Note: sox_precision() returns 0 for SOX_ENCODING_MP2 and _MP3, as it varies
+ * according to whether you're encoding or decoding but sox_ng.c knows
+ * about this and has a special case and reports 24 as the Writes: precision */
+
 /* MAD returns values with MAD_F_FRACBITS (28) bits of precision, though it's
    not certain that all of them are meaningful. Default to 16 bits to
    align with most users expectation of output file should be 16 bits. */
 #define MP3_MAD_PRECISION    16
 
-#ifdef HAVE_MAD_H
+#ifdef HAVE_MAD
 static const char* const mad_library_names[] =
 {
 #ifdef DL_MAD
@@ -165,7 +169,7 @@ static const char* const mad_library_names[] =
   MAD_FUNC(f,x, void, mad_header_init, (struct mad_header *)) \
   MAD_FUNC(f,x, signed long, mad_timer_count, (mad_timer_t, enum mad_units)) \
   MAD_FUNC(f,x, void, mad_timer_multiply, (mad_timer_t *, signed long))
-#endif /* HAVE_MAD_H */
+#endif /* HAVE_MAD */
 
 #if HAVE_LAME
 static const char* const lame_library_names[] =
@@ -235,7 +239,7 @@ static const char* const lame_library_names[] =
   LAME_FUNC_ID3(f,x, size_t, lame_get_id3v2_tag, (lame_global_flags *, unsigned char*, size_t)) \
   LAME_FUNC_ID3(f,x, int, id3tag_set_fieldvalue, (lame_global_flags *, const char *))
 
-#ifdef HAVE_TWOLAME_H
+#ifdef HAVE_TWOLAME
 static const char* const twolame_library_names[] =
 {
 #ifdef DL_TWOLAME
@@ -244,7 +248,7 @@ static const char* const twolame_library_names[] =
 #endif
   NULL
 };
-#endif /* HAVE_TWOLAME_H */
+#endif /* HAVE_TWOLAME */
 
 #ifdef DL_TWOLAME
   #define TWOLAME_FUNC LSX_DLENTRY_DYNAMIC
@@ -254,11 +258,14 @@ static const char* const twolame_library_names[] =
 
 #define TWOLAME_FUNC_ENTRIES(f,x) \
   TWOLAME_FUNC(f,x, twolame_options*, twolame_init, (void)) \
+  TWOLAME_FUNC(f,x, int, twolame_set_verbosity, (twolame_options*, int)) \
   TWOLAME_FUNC(f,x, int, twolame_get_num_channels, (twolame_options*)) \
   TWOLAME_FUNC(f,x, int, twolame_set_num_channels, (twolame_options*, int)) \
   TWOLAME_FUNC(f,x, int, twolame_set_in_samplerate, (twolame_options *, int)) \
   TWOLAME_FUNC(f,x, int, twolame_set_out_samplerate, (twolame_options *, int)) \
   TWOLAME_FUNC(f,x, int, twolame_set_brate, (twolame_options *, int)) \
+  TWOLAME_FUNC(f,x, int, twolame_set_VBR, (twolame_options *, int)) \
+  TWOLAME_FUNC(f,x, int, twolame_set_VBR_level, (twolame_options *, float)) \
   TWOLAME_FUNC(f,x, int, twolame_init_params, (twolame_options *)) \
   TWOLAME_FUNC(f,x, int, twolame_encode_buffer_float32_interleaved, (twolame_options *, const float [], int, unsigned char *, int)) \
   TWOLAME_FUNC(f,x, int, twolame_encode_flush, (twolame_options *, unsigned char *, int)) \
@@ -269,7 +276,7 @@ typedef struct mp3_priv_t {
   unsigned char *mp3_buffer;
   size_t mp3_buffer_size;
 
-#ifdef HAVE_MAD_H
+#ifdef HAVE_MAD
   struct mad_stream       Stream;
   struct mad_frame        Frame;
   struct mad_synth        Synth;
@@ -277,7 +284,7 @@ typedef struct mp3_priv_t {
   ptrdiff_t               cursamp;
   size_t                  FrameCount;
   LSX_DLENTRIES_TO_PTRS(MAD_FUNC_ENTRIES, mad_dl);
-#endif /*HAVE_MAD_H*/
+#endif /* HAVE_MAD */
 
 #if defined(HAVE_LAME) || defined(HAVE_TWOLAME)
   float *pcm_buffer;
@@ -298,7 +305,7 @@ typedef struct mp3_priv_t {
 #endif
 } priv_t;
 
-#if defined(HAVE_MAD_H)
+#ifdef HAVE_MAD 
 
 /* This function merges the functions tagtype() and id3_tag_query()
    from MAD's libid3tag, so we don't have to link to it
@@ -329,11 +336,11 @@ static int tagtype(const unsigned char *data, size_t length)
     return 0;
 }
 
-#endif /* HAVE_MAD_H */
+#endif /* HAVE_MAD */
 
 #include "mp3-util.h"
 
-#ifdef HAVE_MAD_H
+#ifdef HAVE_MAD
 
 /*
  * (Re)fill the stream buffer that is to be decoded.  If any data
@@ -448,8 +455,6 @@ static int startread(sox_format_t * ft)
   p->mad_synth_init(&p->Synth);
   mad_timer_reset(&p->Timer);
 
-  ft->encoding.encoding = SOX_ENCODING_MP3;
-
   /* Decode at least one valid frame to find out the input
    * format.  The decoded frame will be saved off so that it
    * can be processed later.
@@ -464,6 +469,7 @@ static int startread(sox_format_t * ft)
    * that we have a valid MP3 and also skips past ID3v2 tags
    * at the beginning of the audio file.
    */
+
   p->Stream.error = 0;
   while (p->mad_frame_decode(&p->Frame,&p->Stream))
   {
@@ -493,6 +499,32 @@ static int startread(sox_format_t * ft)
       return SOX_EOF;
   }
 
+  lsx_report("MAD says it's MPEG-1 layer %d",
+      p->Frame.header.layer == MAD_LAYER_I ? 1 :
+      p->Frame.header.layer == MAD_LAYER_II ? 2 :
+      p->Frame.header.layer == MAD_LAYER_III ? 3 :
+      p->Frame.header.layer);
+
+  switch (p->Frame.header.layer) {
+  case MAD_LAYER_I:
+      /* MPEG-1 Layer I is a simpler encoding at bitrates from 32 to 448 kbps
+       * and sampling rates of 32, 44.1 and 48 kHz.
+       * It uses a 512-point DFT as opposed to Layer 2's 1024-point DFT
+       * and was used in the Philips Digital Compact Cassette.
+       * MP1 is a subset of MP2 and MP2 decoders can decode MP1 */
+      ft->encoding.encoding = SOX_ENCODING_MP1;
+      break;
+  case MAD_LAYER_II:
+      ft->encoding.encoding = SOX_ENCODING_MP2;
+      break;
+  case MAD_LAYER_III: /* mono or stereo */
+      ft->encoding.encoding = SOX_ENCODING_MP3;
+      break;
+  default:
+      lsx_fail_errno(ft,SOX_EOF,"no valid MPEG layer found");
+      return SOX_EOF;
+  }
+
   switch(p->Frame.header.mode)
   {
       case MAD_MODE_SINGLE_CHANNEL:
@@ -581,7 +613,7 @@ static size_t sox_mp3read(sox_format_t * ft, sox_sample_t *buf, size_t len)
                     continue;
                 else
                 {
-                    lsx_report("unrecoverable frame level error (%s).",
+                    lsx_report("unrecoverable frame level error (%s)",
                               p->mad_stream_errorstr(&p->Stream));
                     break;
                 }
@@ -714,20 +746,17 @@ static int sox_mp3seek(sox_format_t * ft, sox_uint64_t offset)
 
   return SOX_EOF;
 }
-#endif /* !HAVE_MAD_H */
+#endif /* HAVE_MAD */
 
 
+#ifndef HAVE_MAD
 
-#ifndef HAVE_MAD_H
-static int startread(sox_format_t * ft)
-{
-  lsx_fail_errno(ft,SOX_EOF,"SoX was compiled without MP3 decoding support");
-  return SOX_EOF;
-}
+#define startread NULL
 #define sox_mp3read NULL
 #define stopread NULL
 #define sox_mp3seek NULL
-#endif /*HAVE_MAD_H*/
+
+#endif /* !HAVE_MAD */
 
 #ifdef HAVE_LAME
 
@@ -825,10 +854,13 @@ static void rewrite_id3v2_tag(sox_format_t * ft, size_t id3v2_size, uint64_t num
 
   if (LSX_DLFUNC_IS_STUB(p, lame_get_id3v2_tag))
   {
-    if (p->num_samples)
-      lsx_warn("cannot update track length info - tag update not supported with this version of LAME. Track length will be incorrect.");
-    else
-      lsx_report("cannot update track length info - tag update not supported with this version of LAME. Track length will be unspecified.");
+    if (p->num_samples) {
+      lsx_warn("this version of LAME does not support tag update;");
+      lsx_warn("the track length will be incorrect");
+    } else {
+      lsx_report("this version of LAME does not support tag update;");
+      lsx_report("the track length will be unspecified");
+    }
     return;
   }
 
@@ -852,10 +884,13 @@ static void rewrite_id3v2_tag(sox_format_t * ft, size_t id3v2_size, uint64_t num
   if (new_size != id3v2_size) {
     if (LSX_DLFUNC_IS_STUB(p, id3tag_set_pad))
     {
-      if (p->num_samples)
-        lsx_warn("cannot update track length info - tag size adjustment not supported with this version of LAME. Track length will be invalid.");
-      else
-        lsx_report("cannot update track length info - tag size adjustment not supported with this version of LAME. Track length will be unspecified.");
+      if (p->num_samples) {
+        lsx_report("this version of LAME does not support tag size adjustment;");
+        lsx_report("the track length will be invalid");
+      } else {
+        lsx_report("this version of LAME does not support tag size adjustment;");
+        lsx_report("the track length will be unspecified");
+      }
     }
     else
       lsx_warn("cannot update track length info - failed to adjust tag size");
@@ -930,132 +965,132 @@ static void rewrite_tags(sox_format_t * ft, uint64_t num_samples)
 
 #define LAME_BUFFER_SIZE(num_samples) (((num_samples) + 3) / 4 * 5 + 7200)
 
-static int startwrite(sox_format_t * ft)
+static int startwrite_mp2(sox_format_t * ft)
 {
+#if !HAVE_TWOLAME
+  lsx_fail_errno(ft,SOX_EOF,"SoX was compiled without MP2 encoding support");
+  return SOX_EOF;
+#else
   priv_t *p = (priv_t *) ft->priv;
   int openlibrary_result;
-  int fail = 0;
 
-  if (ft->encoding.encoding != SOX_ENCODING_MP3) {
-    if(ft->encoding.encoding != SOX_ENCODING_UNKNOWN)
-      lsx_report("Encoding forced to MP2/MP3");
-    ft->encoding.encoding = SOX_ENCODING_MP3;
-  }
-
-  if(strchr(ft->filetype, '2'))
-      p->mp2 = 1;
-
-  if (p->mp2) {
-#ifdef HAVE_TWOLAME
-    LSX_DLLIBRARY_OPEN(
-        p,
-        twolame_dl,
-        TWOLAME_FUNC_ENTRIES,
-        "Twolame encoder library",
-        twolame_library_names,
-        openlibrary_result);
-#else
-    lsx_fail_errno(ft,SOX_EOF,"SoX was compiled without MP2 encoding support");
-    return SOX_EOF;
-#endif
-  } else {
-#ifdef HAVE_LAME
-    LSX_DLLIBRARY_OPEN(
-        p,
-        lame_dl,
-        LAME_FUNC_ENTRIES,
-        "LAME encoder library",
-        lame_library_names,
-        openlibrary_result);
-#else
-    lsx_fail_errno(ft,SOX_EOF,"SoX was compiled without MP3 encoding support");
-    return SOX_EOF;
-#endif
-  }
+  LSX_DLLIBRARY_OPEN(
+      p,
+      twolame_dl,
+      TWOLAME_FUNC_ENTRIES,
+      "Twolame encoder library",
+      twolame_library_names,
+      openlibrary_result);
   if (openlibrary_result)
     return SOX_EOF;
 
-  p->mp3_buffer_size = LAME_BUFFER_SIZE(sox_globals.bufsiz / max(ft->signal.channels, 1));
-  p->mp3_buffer = lsx_malloc(p->mp3_buffer_size);
-
-  p->pcm_buffer_size = sox_globals.bufsiz * sizeof(float);
-  p->pcm_buffer = lsx_malloc(p->pcm_buffer_size);
-
-  if (p->mp2) {
-#ifdef HAVE_TWOLAME
-    p->opt = p->twolame_init();
-
-    if (p->opt == NULL){
-      lsx_fail_errno(ft,SOX_EOF,"initialization of Twolame library failed");
-      return(SOX_EOF);
-    }
-#endif
-  } else {
-#ifdef HAVE_LAME
-    p->gfp = p->lame_init();
-
-    if (p->gfp == NULL){
-      lsx_fail_errno(ft,SOX_EOF,"initialization of LAME library failed");
-      return(SOX_EOF);
-    }
-
-    /* First set message callbacks so we don't miss any messages: */
-    p->lame_set_errorf(p->gfp,errorf);
-    p->lame_set_debugf(p->gfp,debugf);
-    p->lame_set_msgf  (p->gfp,msgf);
-
-    p->num_samples = ft->signal.length == SOX_IGNORE_LENGTH ? 0 : ft->signal.length / max(ft->signal.channels, 1);
-    p->lame_set_num_samples(p->gfp, p->num_samples > ULONG_MAX ? 0 : (unsigned long)p->num_samples);
-#endif
+  if ((p->opt = p->twolame_init()) == NULL){
+    lsx_fail_errno(ft,SOX_EOF,"initialization of Twolame library failed");
+    return SOX_EOF;
   }
+  (void) p->twolame_set_verbosity(p->opt, 0);
+
+  ft->signal.precision = MP2_TWOLAME_PRECISION;
+
+  if (ft->signal.channels != SOX_ENCODING_UNKNOWN) {
+    if (p->twolame_set_num_channels(p->opt,(int)ft->signal.channels) != 0) {
+      lsx_fail_errno(ft,SOX_EOF,"unsupported number of channels");
+      return(SOX_EOF);
+    }
+  } else {
+    ft->signal.channels = p->twolame_get_num_channels(p->opt); /* Twolame default */
+  }
+
+  p->twolame_set_in_samplerate(p->opt,(int)ft->signal.rate);
+  p->twolame_set_out_samplerate(p->opt,(int)ft->signal.rate);
+
+  lsx_debug("-C option is %f", ft->encoding.compression);
+
+  if (ft->encoding.compression == HUGE_VAL) {
+    /* Do nothing, use defaults: */
+    lsx_report("using MP2 encoding defaults");
+  } else {
+    /* Twolame's CBR rates are exactly 32,48,64... and VBR rates a float
+     * from -50 to +50 with "useful" values from -10 to +10 so we count
+     * any floating point value from -50 to 50 as VBR unless it is exactly
+     * 32 or 48, and anything else as CBR.
+     */
+    if (ft->encoding.compression >=-50 && ft->encoding.compression <= 50 &&
+        ft->encoding.compression != 32 && ft->encoding.compression != 48) {
+        if (p->twolame_set_VBR(p->opt, 1) != 0) {
+          lsx_fail_errno(ft,SOX_EOF,"failed to enable MP2 VBR");
+          return(SOX_EOF);
+        }
+        if (p->twolame_set_VBR_level(p->opt, ft->encoding.compression) != 0) {
+          lsx_fail_errno(ft,SOX_EOF,"failed to set MP2 VBR level");
+          return(SOX_EOF);
+        }
+    } else {
+      if (p->twolame_set_brate(p->opt, (int)ft->encoding.compression) != 0) {
+        lsx_fail_errno(ft, SOX_EOF, "invalid MP2 bitrate");
+        return(SOX_EOF);
+      }
+      lsx_report("twolame set bitrate(%d)", (int)ft->encoding.compression);
+    }
+  }
+
+  if (p->twolame_init_params(p->opt) != 0) {
+    lsx_fail_errno(ft,SOX_EOF,"Twolame initialization failed");
+    return(SOX_EOF);
+  }
+
+  return(SOX_SUCCESS);
+#endif
+}
+
+static int startwrite_mp3(sox_format_t * ft)
+{
+#if !HAVE_LAME
+  lsx_fail_errno(ft,SOX_EOF,"SoX was compiled without MP3 encoding support");
+  return SOX_EOF;
+#else
+  priv_t *p = (priv_t *) ft->priv;
+  int openlibrary_result;
+
+  LSX_DLLIBRARY_OPEN(
+      p,
+      lame_dl,
+      LAME_FUNC_ENTRIES,
+      "LAME encoder library",
+      lame_library_names,
+      openlibrary_result);
+  if (openlibrary_result)
+    return SOX_EOF;
+
+  if ((p->gfp = p->lame_init()) == NULL){
+    lsx_fail_errno(ft,SOX_EOF,"initialization of LAME library failed");
+    return SOX_EOF;
+  }
+
+  /* First set message callbacks so we don't miss any messages: */
+  p->lame_set_errorf(p->gfp,errorf);
+  p->lame_set_debugf(p->gfp,debugf);
+  p->lame_set_msgf  (p->gfp,msgf);
+
+  p->num_samples = ft->signal.length == SOX_IGNORE_LENGTH ? 0 : ft->signal.length / max(ft->signal.channels, 1);
+  p->lame_set_num_samples(p->gfp, p->num_samples > ULONG_MAX ? 0 : (unsigned long)p->num_samples);
 
   ft->signal.precision = MP3_LAME_PRECISION;
 
   if (ft->signal.channels != SOX_ENCODING_UNKNOWN) {
-    if (p->mp2) {
-#ifdef HAVE_TWOLAME
-      fail = (p->twolame_set_num_channels(p->opt,(int)ft->signal.channels) != 0);
-#endif
-    } else {
-#ifdef HAVE_LAME
-      fail = (p->lame_set_num_channels(p->gfp,(int)ft->signal.channels) < 0);
-#endif
-    }
-    if (fail) {
+    if (p->lame_set_num_channels(p->gfp,(int)ft->signal.channels) < 0) {
       lsx_fail_errno(ft,SOX_EOF,"unsupported number of channels");
       return(SOX_EOF);
     }
-  }
-  else {
-    if (p->mp2) {
-#ifdef HAVE_TWOLAME
-      ft->signal.channels = p->twolame_get_num_channels(p->opt); /* Twolame default */
-#endif
-    } else {
-#ifdef HAVE_LAME
-      ft->signal.channels = p->lame_get_num_channels(p->gfp); /* LAME default */
-#endif
-    }
-  }
-
-  if (p->mp2) {
-#ifdef HAVE_TWOLAME
-    p->twolame_set_in_samplerate(p->opt,(int)ft->signal.rate);
-    p->twolame_set_out_samplerate(p->opt,(int)ft->signal.rate);
-#endif
   } else {
-#ifdef HAVE_LAME
-    p->lame_set_in_samplerate(p->gfp,(int)ft->signal.rate);
-    p->lame_set_out_samplerate(p->gfp,(int)ft->signal.rate);
-#endif
+    ft->signal.channels = p->lame_get_num_channels(p->gfp); /* LAME default */
   }
 
-  if (!p->mp2) {
-#ifdef HAVE_LAME
-    if (!LSX_DLFUNC_IS_STUB(p, id3tag_init))
-      write_comments(ft);
-#endif
-  }
+  p->lame_set_in_samplerate(p->gfp,(int)ft->signal.rate);
+  p->lame_set_out_samplerate(p->gfp,(int)ft->signal.rate);
+
+  if (!LSX_DLFUNC_IS_STUB(p, id3tag_init))
+    write_comments(ft);
 
   /* The primary parameter to the LAME encoder is the bit rate. If the
    * value of encoding.compression is a positive integer, it's taken as
@@ -1090,13 +1125,13 @@ static int startwrite(sox_format_t * ft)
    * Note: It would have been nice to simply use low values, 0-9, to trigger
    * VBR mode, but 8 kbps is a valid bit rate, so negative values were
    * used instead.
-  */
+   */
 
   lsx_debug("-C option is %f", ft->encoding.compression);
 
   if (ft->encoding.compression == HUGE_VAL) {
     /* Do nothing, use defaults: */
-    lsx_report("using %s encoding defaults", p->mp2? "MP2" : "MP3");
+    lsx_report("using MP3 encoding defaults");
   } else {
     double abs_compression = fabs(ft->encoding.compression);
     double floor_compression = floor(abs_compression);
@@ -1108,11 +1143,6 @@ static int startwrite(sox_format_t * ft)
         : (int)(fraction_compression * 10.0 + 0.5);
 
     if (ft->encoding.compression < 0.5) {
-      if (p->mp2) {
-        lsx_fail_errno(ft,SOX_EOF,"variable bitrate encoding not supported for MP2 audio");
-        return(SOX_EOF);
-      }
-#ifdef HAVE_LAME
       if (p->lame_get_VBR(p->gfp) == vbr_off)
         p->lame_set_VBR(p->gfp, vbr_default);
 
@@ -1130,63 +1160,62 @@ static int startwrite(sox_format_t * ft)
         return(SOX_EOF);
       }
       lsx_report("lame_set_VBR_q(%d)", bitrate_q);
-#endif
     } else {
-      if (p->mp2) {
-#ifdef HAVE_TWOLAME
-        fail = (p->twolame_set_brate(p->opt, bitrate_q) != 0);
-#endif
-      } else {
-#ifdef HAVE_LAME
-        fail = (p->lame_set_brate(p->gfp, bitrate_q) < 0);
-#endif
-      }
-      if (fail) {
+      if (p->lame_set_brate(p->gfp, bitrate_q) < 0) {
         lsx_fail_errno(ft, SOX_EOF,
-          "%slame_set_brate(%d) failed", p->mp2? "two" : "", bitrate_q);
+          "lame_set_brate(%d) failed", bitrate_q);
         return(SOX_EOF);
       }
-      lsx_report("(two)lame_set_brate(%d)", bitrate_q);
+      lsx_report("lame_set_brate(%d)", bitrate_q);
     }
 
     /* Set Quality */
 
-    if (encoder_q < 0 || p->mp2) {
+    if (encoder_q < 0) {
       /* use default quality value */
-      lsx_report("using %s default quality", p->mp2? "MP2" : "MP3");
+      lsx_report("using MP3 default quality");
     } else {
-#ifdef HAVE_LAME
       if (p->lame_set_quality(p->gfp, encoder_q) < 0) {
         lsx_fail_errno(ft, SOX_EOF,
           "lame_set_quality(%d) failed", encoder_q);
         return(SOX_EOF);
       }
       lsx_report("lame_set_quality(%d)", encoder_q);
-#endif
     }
   }
 
-  if (!p->mp2) {
-#ifdef HAVE_LAME
-    p->lame_set_bWriteVbrTag(p->gfp, p->vbr_tag);
-#endif
-  }
+  p->lame_set_bWriteVbrTag(p->gfp, p->vbr_tag);
 
-  if (p->mp2) {
-#ifdef HAVE_TWOLAME
-    fail = (p->twolame_init_params(p->opt) != 0);
-#endif
-  } else {
-#ifdef HAVE_LAME
-    fail = (p->lame_init_params(p->gfp) < 0);
-#endif
-  }
-  if (fail) {
-    lsx_fail_errno(ft,SOX_EOF,"%s initialization failed", p->mp2? "Twolame" : "LAME");
+  if (p->lame_init_params(p->gfp) < 0) {
+    lsx_fail_errno(ft,SOX_EOF,"LAME initialization failed");
     return(SOX_EOF);
   }
 
   return(SOX_SUCCESS);
+#endif
+}
+
+static int startwrite(sox_format_t * ft)
+{
+  priv_t *p = (priv_t *) ft->priv;
+
+  p->mp2 = !!strchr(ft->filetype, '2');
+
+  if (ft->encoding.encoding != SOX_ENCODING_MP2 &&
+      ft->encoding.encoding != SOX_ENCODING_MP3) {
+    if(ft->encoding.encoding != SOX_ENCODING_UNKNOWN)
+      lsx_report("encoding forced to MP%d", p->mp2 ? 2 : 3);
+    ft->encoding.encoding = p->mp2 ? SOX_ENCODING_MP2 : SOX_ENCODING_MP3;
+  }
+
+  p->mp3_buffer_size = LAME_BUFFER_SIZE(sox_globals.bufsiz / max(ft->signal.channels, 1));
+  p->mp3_buffer = lsx_malloc(p->mp3_buffer_size);
+
+  p->pcm_buffer_size = sox_globals.bufsiz * sizeof(float);
+  p->pcm_buffer = lsx_malloc(p->pcm_buffer_size);
+
+
+  return p->mp2 ? startwrite_mp2(ft) : startwrite_mp3(ft);
 }
 
 #define MP3_SAMPLE_TO_FLOAT(d,clips) ((float)(32768*SOX_SAMPLE_TO_FLOAT_32BIT(d,clips)))
@@ -1319,28 +1348,57 @@ static int stopwrite(sox_format_t * ft)
 }
 
 #else /* !(HAVE_LAME || HAVE_TWOLAME) */
-static int startwrite(sox_format_t * ft UNUSED)
-{
-  lsx_fail_errno(ft,SOX_EOF,"SoX was compiled with neither MP2 nor MP3 encoding support");
-  return SOX_EOF;
-}
+
+#define startwrite NULL
 #define sox_mp3write NULL
 #define stopwrite NULL
+
 #endif /* HAVE_LAME || HAVE_TWOLAME */
 
-LSX_FORMAT_HANDLER(mp3)
+/* MAD can tell the difference between Layer 1 and Layer 2 but decodes them both
+ * with the same decoder (Layer 1 is a subset of Layer 2) but soxi might
+ * tell people if they happen across an MP1 file */
+LSX_FORMAT_HANDLER(mp1)
 {
-  static char const * const names[] = {"mp3", "mp2", "audio/mpeg", NULL};
-  static unsigned const write_encodings[] = {
-    SOX_ENCODING_MP3, 0, 0};
-  static sox_rate_t const write_rates[] = {
-    8000, 11025, 12000, 16000, 22050, 24000, 32000, 44100, 48000, 0};
+  static char const * const names[] = {"mp1", "m1a", NULL};
   static sox_format_handler_t const handler = {SOX_LIB_VERSION_CODE,
-    "MPEG Layer 2/3 lossy audio compression", names, 0,
+    "MPEG-1 Layer 1 lossy audio compression", names, 0,
+    startread, sox_mp3read, stopread,
+    NULL, NULL, NULL,
+    sox_mp3seek, NULL, NULL, sizeof(priv_t)
+  };
+  return &handler;
+}
+
+LSX_FORMAT_HANDLER(mp2)
+{
+  static char const * const names[] = {"mp2", NULL};
+  static unsigned const write_encodings[] = {
+    SOX_ENCODING_MP2, 0, 0};
+  static sox_rate_t const write_rates[] = {
+    16000, 22050, 24000, 32000, 44100, 48000, 0};
+  static sox_format_handler_t const handler = {SOX_LIB_VERSION_CODE,
+    "MPEG-1 Layer 2 lossy audio compression", names, 0,
     startread, sox_mp3read, stopread,
     startwrite, sox_mp3write, stopwrite,
     sox_mp3seek, write_encodings, write_rates, sizeof(priv_t)
   };
   return &handler;
 }
-#endif /* defined(HAVE_MAD_H) || defined(HAVE_LAME) || defined(HAVE_TWOLAME) */
+
+LSX_FORMAT_HANDLER(mp3)
+{
+  static char const * const names[] = {"mp3", NULL};
+  static unsigned const write_encodings[] = {
+    SOX_ENCODING_MP3, 0, 0};
+  static sox_rate_t const write_rates[] = {
+    8000, 11025, 12000, 16000, 22050, 24000, 32000, 44100, 48000, 0};
+  static sox_format_handler_t const handler = {SOX_LIB_VERSION_CODE,
+    "MPEG-1 Layer 3 lossy audio compression", names, 0,
+    startread, sox_mp3read, stopread,
+    startwrite, sox_mp3write, stopwrite,
+    sox_mp3seek, write_encodings, write_rates, sizeof(priv_t)
+  };
+  return &handler;
+}
+#endif /* defined(HAVE_MAD) || defined(HAVE_LAME) || defined(HAVE_TWOLAME) */
